@@ -22,34 +22,35 @@
  * default folders are updated as needed.
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
-
-#include "exchange-config-listener.h"
-
-#include <exchange-account.h>
-#include <e-folder-exchange.h>
-#include <e2k-marshal.h>
-#include <e2k-uri.h>
-#include "mail-stub-listener.h"
-
-#include <e-util/e-dialog-utils.h>
-
-#include <libedataserver/e-source.h>
-#include <libedataserver/e-source-list.h>
-#include <libedataserver/e-source-group.h>
-#include <libedataserver/e-xml-hash-utils.h>
-
-#include <camel/camel-url.h>
 
 #include <sys/types.h>
-#include <dirent.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include <glib.h>
+#include <glib/gstdio.h>
+
+#include <libedataserver/e-source.h>
+#include <libedataserver/e-source-list.h>
+#include <libedataserver/e-source-group.h>
+#include <libedataserver/e-xml-utils.h>
+#include <libedataserver/e-xml-hash-utils.h>
+
+#include <camel/camel-url.h>
+
+#include <e2k-marshal.h>
+#include <e2k-uri.h>
+#include <exchange-account.h>
+#include <e-folder-exchange.h>
+
+#include <e-util/e-dialog-utils.h>
+
+#include "mail-stub-listener.h"
+
+#include "exchange-config-listener.h"
 
 struct _ExchangeConfigListenerPrivate {
 	GConfClient *gconf;
@@ -92,7 +93,7 @@ static void account_changed (EAccountList *account_listener,
 static void account_removed (EAccountList *account_listener,
 			     EAccount     *account);
 
-static gboolean exchange_camel_urls_is_equal (const gchar *url1, 
+static gboolean exchange_camel_urls_is_equal (const gchar *url1,
 					      const gchar *url2);
 
 
@@ -119,7 +120,7 @@ class_init (GObjectClass *object_class)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (ExchangeConfigListenerClass, exchange_account_created),
 			      NULL, NULL,
-			      e2k_marshal_NONE__POINTER,
+			      g_cclosure_marshal_VOID__POINTER,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_POINTER);
 	signals[EXCHANGE_ACCOUNT_REMOVED] =
@@ -128,7 +129,7 @@ class_init (GObjectClass *object_class)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (ExchangeConfigListenerClass, exchange_account_removed),
 			      NULL, NULL,
-			      e2k_marshal_NONE__POINTER,
+			      g_cclosure_marshal_VOID__POINTER,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_POINTER);
 }
@@ -190,22 +191,21 @@ is_active_exchange_account (EAccount *account)
 	return (strncmp (account->source->url, EXCHANGE_URI_PREFIX, 11) == 0);
 }
 
-static void 
+static void
 update_foreign_uri (const char *path, const char *account_uri)
 {
 	char *file_path, *phy_uri, *foreign_uri, *new_phy_uri;
-	struct stat file_stat;
 	GHashTable *old_props = NULL;
-	xmlDoc *old_doc, *new_doc = NULL;
+	xmlDoc *old_doc = NULL, *new_doc = NULL;
 
 	if (!path)
 		return;
 
 	file_path = g_build_filename (path, "hierarchy.xml", NULL);
-	if (stat (file_path, &file_stat) < 0)
+	if (!g_file_test (file_path, G_FILE_TEST_EXISTS))
 		goto cleanup;
 
-	old_doc = xmlParseFile (file_path);
+	old_doc = e_xml_parse_file (file_path);
 	if (!old_doc)
 		goto cleanup;
 
@@ -231,34 +231,34 @@ update_foreign_uri (const char *path, const char *account_uri)
 	g_hash_table_insert (old_props, (char *)g_strdup ("physical_uri_prefix"), new_phy_uri);
 
 	new_doc = e_xml_from_hash (old_props, E_XML_HASH_TYPE_PROPERTY, "foreign-hierarchy");
-	xmlSaveFile (file_path, new_doc);
+	e_xml_save_file (file_path, new_doc);
 
 	xmlFreeDoc (new_doc);
 	g_free (new_phy_uri);
 cleanup:
 	g_free (file_path);
-	g_hash_table_destroy (old_props);
+	e_xml_destroy_hash (old_props);
 	return;
 }
 
 static void
 migrate_foreign_hierarchy (ExchangeAccount *account)
 {
-	DIR *d;
-	struct dirent *dentry;
+	GDir *d;
+	const char *dentry;
 	char *dir;
 
-	d = opendir (account->storage_dir);
+	d = g_dir_open (account->storage_dir, 0, NULL);
 	if (d) {
-		while ((dentry = readdir (d))) {
-			if (!strchr (dentry->d_name, '@'))
+		while ((dentry = g_dir_read_name (d))) {
+			if (!strchr (dentry, '@'))
 				continue;
-			dir = g_strdup_printf ("%s/%s", account->storage_dir, 
-							dentry->d_name);
+			dir = g_strdup_printf ("%s/%s", account->storage_dir,
+							dentry);
 			update_foreign_uri (dir, account->account_filename);
 			g_free (dir);
 		}
-		closedir (d);
+		g_dir_close (d);
 	}
 }
 
@@ -276,15 +276,16 @@ ex_set_relative_uri (ESource *source, const char *url)
 	if (!folder_name)
 		return;
 
-	if ((folder_name + 1) && *(folder_name + 1) == ';') 
+	if ((folder_name + 1) && *(folder_name + 1) == ';')
 		return;
 
 	new_rel_uri = g_strdup_printf ("%s;%s", url, folder_name + 1);
 	e_source_set_relative_uri (source, new_rel_uri);
+	g_free (new_rel_uri);
 }
 
-static void 
-migrate_account_esource (EAccount *account, 
+static void
+migrate_account_esource (EAccount *account,
 		        FolderType folder_type)
 {
 	ESourceGroup *group;
@@ -305,16 +306,19 @@ migrate_account_esource (EAccount *account,
 	authtype  = camel_url->authmech;
 	url_string = camel_url_to_string (camel_url, CAMEL_URL_HIDE_PASSWORD | CAMEL_URL_HIDE_PARAMS);
 
-	if (!user_name) 
+	if (!user_name) {
+		g_free (url_string);
+		camel_url_free (camel_url);
 		return;
+	}
 
 	client = gconf_client_get_default ();
 
 	if (folder_type == EXCHANGE_CONTACTS_FOLDER)
-		source_list = e_source_list_new_for_gconf ( client, 
+		source_list = e_source_list_new_for_gconf ( client,
 							CONF_KEY_CONTACTS);
 	else if (folder_type == EXCHANGE_CALENDAR_FOLDER)
-		source_list = e_source_list_new_for_gconf ( client, 
+		source_list = e_source_list_new_for_gconf ( client,
 							CONF_KEY_CAL);
 	else if (folder_type == EXCHANGE_TASKS_FOLDER)
 		source_list = e_source_list_new_for_gconf ( client,
@@ -348,6 +352,8 @@ migrate_account_esource (EAccount *account,
 			}
 		}
 	}
+	g_free (url_string);
+	camel_url_free (camel_url);
 	g_object_unref (source_list);
 	g_object_unref (client);
 }
@@ -355,10 +361,11 @@ migrate_account_esource (EAccount *account,
 void
 exchange_config_listener_migrate_esources (ExchangeConfigListener *config_listener)
 {
+	EAccount *account;
+
 	g_return_if_fail (config_listener != NULL);
 
-	EAccount *account = config_listener->priv->configured_account;
-
+	account = config_listener->priv->configured_account;
 	migrate_account_esource (account, EXCHANGE_CALENDAR_FOLDER);
 	migrate_account_esource (account, EXCHANGE_TASKS_FOLDER);
 	migrate_account_esource (account, EXCHANGE_CONTACTS_FOLDER);
@@ -438,7 +445,7 @@ requires_relogin (char *current_url, char *new_url)
 		relogin = TRUE;
 		goto end;
 	}
-	
+
 	if (current_uri->authmech || new_uri->authmech) {
 		if (current_uri->authmech && new_uri->authmech) {
 	    		if (strcmp (current_uri->authmech, new_uri->authmech)) {
@@ -446,7 +453,7 @@ requires_relogin (char *current_url, char *new_url)
 				relogin = TRUE;
 				goto end;
 			}
-		} 
+		}
 		else {
 			/* Auth mechanism is set for the first time */
 			relogin = TRUE;
@@ -454,10 +461,10 @@ requires_relogin (char *current_url, char *new_url)
 		}
 	}
 
-	for (i=0; i<n_params; i++) { 
+	for (i=0; i<n_params; i++) {
 		current_param_val = e2k_uri_get_param (current_uri, params[i]);
-		new_param_val = e2k_uri_get_param (new_uri, params[i]); 
-	
+		new_param_val = e2k_uri_get_param (new_uri, params[i]);
+
 		if (current_param_val && new_param_val) {
 			/* both the urls have params to be compared */
 			if (strcmp (current_param_val, new_param_val)) {
@@ -503,11 +510,11 @@ account_changed (EAccountList *account_list, EAccount *account)
 		return;
 	}
 
-	/* FIXME: The order of the parameters in the Camel URL string is not in 
+	/* FIXME: The order of the parameters in the Camel URL string is not in
 	 * order for the two given strings. So, we will not be able to use
 	 * plain string comparison. Instead compare the parameters one by one.
 	 */
-	if (exchange_camel_urls_is_equal (config_listener->priv->configured_uri, 
+	if (exchange_camel_urls_is_equal (config_listener->priv->configured_uri,
 					  account->source->url) &&
 	    !strcmp (config_listener->priv->configured_name, account->name)) {
 		/* The user changed something we don't care about. */
@@ -538,11 +545,11 @@ account_changed (EAccountList *account_list, EAccount *account)
 		return;
 	}
 
-	/* If account name has changed, or the url value has changed, which 
-	 * could be due to change in hostname or some parameter value, 
-	 * remove old e-sources 
+	/* If account name has changed, or the url value has changed, which
+	 * could be due to change in hostname or some parameter value,
+	 * remove old e-sources
 	 */
-	if (requires_relogin (config_listener->priv->configured_uri, 
+	if (requires_relogin (config_listener->priv->configured_uri,
 			      account->source->url)) {
 		exchange_account_forget_password (priv->exchange_account);
 	} else if (strcmp (config_listener->priv->configured_name, account->name)) {
@@ -581,15 +588,12 @@ account_removed (EAccountList *account_list, EAccount *account)
 			       signals[EXCHANGE_ACCOUNT_REMOVED], 0,
 			       priv->exchange_account);
 
-		g_object_unref (priv->exchange_account);
-		priv->exchange_account = NULL;
-
 		priv->configured_account = NULL;
 		g_free (priv->configured_uri);
 		priv->configured_uri = NULL;
 		g_free (priv->configured_name);
 		priv->configured_name = NULL;
-	} 
+	}
 }
 
 static gboolean
@@ -645,8 +649,8 @@ exchange_config_listener_get_accounts (ExchangeConfigListener *config_listener)
 }
 
 /**
- * exchange_camel_urls_is_equal 
- * 
+ * exchange_camel_urls_is_equal
+ *
  * @url1: CAMEL URL string 1
  * @url2: CAMEL URL string 2
  *
@@ -655,12 +659,12 @@ exchange_config_listener_get_accounts (ExchangeConfigListener *config_listener)
  *
  * Return Value: Boolean result of the comparision.
  *
- **/ 
+ **/
 static gboolean
 exchange_camel_urls_is_equal (const gchar *url1, const gchar *url2)
 {
 	CamelURL *curl1, *curl2;
-	gchar *param1, *param2;
+	const char *param1, *param2;
 	const char *params[] = {
 		"auth",
 		"owa_url",
@@ -670,7 +674,7 @@ exchange_camel_urls_is_equal (const gchar *url1, const gchar *url2)
 	};
 	const int n_params = 5;
 	int i;
-	
+
 	curl1 = camel_url_new (url1, NULL);
 	curl2 = camel_url_new (url2, NULL);
 
@@ -679,16 +683,12 @@ exchange_camel_urls_is_equal (const gchar *url1, const gchar *url2)
 		param2 = (gchar*) camel_url_get_param (curl2, params[i]);
 		if ((param1 && !param2) || (!param1 && param2) || /* Missing */
 		    (param1 && param2 && strcmp (param1, param2))) { /* Differing */
-			g_free (param1);
-			g_free (param2);
-			g_free (curl1);
-			g_free (curl2);
+			camel_url_free (curl1);
+			camel_url_free (curl2);
 			return FALSE;
-		}		
-		g_free (param1);
-		g_free (param2);
+		}
 	}
-	g_free (curl1);
-	g_free (curl2);
+	camel_url_free (curl1);
+	camel_url_free (curl2);
 	return TRUE;
 }
